"""Test kisee quickstart
"""

import io
from test.support import temp_cwd

from _pytest.monkeypatch import MonkeyPatch
import pytest

from kisee import quickstart


def test_quickstart(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("\n" * 1000))
    with temp_cwd():
        quickstart.main()
        with open("settings.toml") as first_settings_file:
            first_settings = first_settings_file.read()
        quickstart.main()
        with open("settings.toml") as second_settings_file:
            second_settings = second_settings_file.read()
        assert first_settings != second_settings


def test_input(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("youpi"))
    assert quickstart.input_or_default("Say youpi?") == "youpi"


def test_good_boolean_input_true(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("y"))
    assert quickstart.input_or_default("Hello", "N", quickstart.boolean) is True


def test_bad_then_good_boolean_input_true(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("pouette\ny"))
    assert quickstart.input_or_default("Hello", "N", quickstart.boolean) is True


def test_good_boolean_input_false(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("n"))
    assert quickstart.input_or_default("Hello", "N", quickstart.boolean) is False


def test_bad_then_good_boolean_input_false(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("pouette\nn"))
    assert quickstart.input_or_default("Hello", "N", quickstart.boolean) is False


def test_default_boolean_input(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr("sys.stdin", io.StringIO("\n"))
    assert quickstart.input_or_default("Hello", "N", quickstart.boolean) is False


def test_bad_boolean_input(monkeypatch: MonkeyPatch) -> None:
    with pytest.raises(ValueError):
        quickstart.boolean("world")



"""This is a really dumb identification backend: it does not store
anything and accepts almost any login/password pair.
"""
from typing import Optional

from kisee.identity_provider import IdentityProvider, User


class DemoBackend(IdentityProvider):
    """Dumb identity backend, for demo purposes.
    This backend follow the following rules:
     - Any user exist and have virtually all passwords.
     - Any password less or equal than 4 characters will fail.
     - root is a superuser.
    Yes, this mean than anybody logging as root with any password of
    more than 4 chars will be superuser. This is for demo purposes only.
    """

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_value, traceback):
        pass

    async def identify(self, login: str, password: str) -> Optional[User]:
        """Identifies the given login/password pair, returns a dict if found.
        """
        # pylint: disable=unused-argument
        if len(password) < 4:
            return None
        return User(
            user_id=login,
            username=login,
            email=f"{login}@example.com",
            is_superuser=login == "root",
        )

    async def register_user(
        self, username: str, password: str, email: str, is_superuser: bool = False
    ):
        pass

    async def get_user_by_email(self, email):
        """Get user with provided email address
        """
        return User(user_id=email, username=email, email=email)

    async def get_user_by_username(self, username):
        """Get user with provided username
        """
        return User(user_id=username, username=username, email=f"{username}@gmail.com")

    async def set_password_for_user(self, user: User, password: str):
        pass

    async def is_connection_alive(self) -> bool:
        """Verify that connection is alive, always return True
        """
        return True



"""Entry point for the identification provider.
"""

import argparse
import logging
import os
import sys
from typing import Mapping, Any

import toml
from aiohttp import web

import sentry_sdk

from kisee import views
from kisee.identity_provider import import_idp
from kisee.middlewares import verify_input_body_is_json, coreapi_error_middleware

Settings = Mapping[str, Any]

AIOHTTP_LOGGERS = (
    "aiohttp.access",
    "aiohttp.client",
    "aiohttp.internal",
    "aiohttp.server",
    "aiohttp.web",
    "aiohttp.websocket",
)


def setup_logging(loglevel):  # pragma: no cover
    """Setup basic logging
    Args:
      loglevel (int): minimum loglevel for emitting messages
    """
    logformat = "[%(asctime)s] %(levelname)s:%(name)s:%(message)s"
    logging.basicConfig(
        level=50 - (loglevel * 10),
        stream=sys.stdout,
        format=logformat,
        datefmt="%Y-%m-%d %H:%M:%S",
    )


def load_conf(settings_path: str) -> Settings:
    """Search for a settings.toml file and load it.
    """
    candidates = (
        settings_path,
        os.path.join(os.getcwd(), settings_path),
        os.path.join(os.getcwd(), "settings.toml"),
        os.path.expanduser("~/settings.toml"),
        os.path.expanduser(os.path.join("~/", settings_path)),
        "/etc/settings.toml",
        os.path.join("/etc/", settings_path),
    )
    for candidate in candidates:
        if os.path.exists(candidate):
            with open(candidate) as candidate_file:
                return toml.load(candidate_file)
    print("Failed to locate the settings.toml file.", file=sys.stderr)
    sys.exit(1)


def parse_args(program_args=None) -> argparse.Namespace:
    """Parses command line arguments.
    """
    if program_args is None:
        program_args = sys.argv[1:]
    parser = argparse.ArgumentParser(description="Shape Identity Provider")
    parser.add_argument("--settings", default="settings.toml")
    parser.add_argument(
        "-v",
        "--verbose",
        dest="loglevel",
        default=0,
        help="Verbose mode (-vv for more, -vvv, …)",
        action="count",
    )
    return parser.parse_args(program_args)


def identification_app(settings: Settings) -> web.Application:
    """Identification provider entry point: builds and run a webserver.
    """
    app = web.Application(
        middlewares=[verify_input_body_is_json, coreapi_error_middleware],
        debug=settings["server"].get("debug", False),
    )
    app["settings"] = settings
    app["identity_backend"] = import_idp(settings["identity_backend"]["class"])(
        settings["identity_backend"].get("options", {})
    )

    async def on_startup_wrapper(app):
        """Wrapper to call __aenter__.
        """
        await app["identity_backend"].__aenter__()

    async def on_cleanup_wrapper(app):
        """Wrapper to call __exit__.
        """
        await app["identity_backend"].__aexit__(None, None, None)

    app.on_startup.append(on_startup_wrapper)
    app.on_cleanup.append(on_cleanup_wrapper)

    app.add_routes(
        [
            web.get("/", views.get_root),
            web.get("/users/", views.get_users),
            web.post("/users/", views.post_users),
            web.get("/jwt/", views.get_jwts),
            web.post("/jwt/", views.post_jwt),
            web.get("/jwt/{jid}", views.get_jwt),
            web.get("/users/", views.get_users),
            web.patch("/users/{username}/", views.patch_user),
            web.get("/forgotten_passwords/", views.get_forgotten_passwords),
            web.post("/forgotten_passwords/", views.post_forgotten_passwords),
            web.get("/health/", views.get_health),
        ]
    )

    return app


def main() -> None:  # pragma: no cover
    """Command line entry point.
    """
    args = parse_args()
    setup_logging(args.loglevel)
    settings = load_conf(args.settings)
    sentry_sdk.init(settings.get("SENTRY_DSN"))
    app = identification_app(settings)
    web.run_app(
        app, host=settings["server"]["host"], port=int(settings["server"]["port"])
    )



"""Abstract class representing an identity provider
"""
from abc import ABC, abstractmethod
from importlib import import_module
from typing import AsyncContextManager, Optional, Type


class UserAlreadyExist(Exception):
    """Exception raised when user already exists
    """


class User:
    """Represents a logged-in, correctly identified, person.
    """

    def __init__(
        self, user_id: str, username: str, email: str, is_superuser: bool = False
    ) -> None:
        self.user_id = user_id
        self.username = username
        self.email = email
        self.is_superuser = is_superuser


class IdentityProvider(
    AsyncContextManager, ABC
):  # pragma: no cover, pylint: disable=inherit-non-class
    # See: https://github.com/PyCQA/pylint/issues/2472
    """Abtract class representing an identity provider
    """

    def __init__(self, options: dict = None) -> None:
        self.options = options
        super().__init__()

    @abstractmethod
    async def identify(self, login: str, password: str) -> Optional[User]:
        """Identifies the given login/password pair, returns a dict if found.
        """

    @abstractmethod
    async def register_user(
        self, username: str, password: str, email: str, is_superuser: bool = False
    ):
        """Create user with login, password and email
        """

    @abstractmethod
    async def get_user_by_email(self, email) -> User:
        """Get user with provided email address
        """

    @abstractmethod
    async def get_user_by_username(self, username) -> User:
        """Get user with provided username
        """

    @abstractmethod
    async def set_password_for_user(self, user: User, password: str):
        """Set password for user
        """

    @abstractmethod
    async def is_connection_alive(self) -> bool:
        """Verify that connection with identity provider datastore is alive
        """


def import_idp(dotted_path: str) -> Type[IdentityProvider]:
    """Import a dotted module path and return the attribute/class
    designated by the last name in the path. Raise ImportError if the
    import failed.
    """
    try:
        module_path, class_name = dotted_path.rsplit(".", 1)
    except ValueError as err:
        raise ImportError("%s doesn't look like a module path" % dotted_path) from err

    module = import_module(module_path)

    try:
        return getattr(module, class_name)
    except AttributeError as err:
        raise ImportError(
            'Module "%s" does not define a "%s" attribute/class'
            % (module_path, class_name)
        ) from err



"""Email utils
"""
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Tuple


def is_email(email: str) -> bool:
    """Assert that email has minimun requirements, can be splited in two parts with '@'
    """
    return "@" in email


def send_mail(subject: str, text: str, html: str, email_settings: dict, recipient: str):
    """Basically send a multipart/alternative email with text and HTML to
    recipient
    """
    msg = MIMEMultipart("alternative")
    msg["Subject"] = subject
    msg["From"] = email_settings["sender"]
    msg.add_header("reply-to", email_settings["sender"])
    msg["To"] = recipient
    msg.attach(MIMEText(text, "plain"))
    msg.attach(MIMEText(html, "html"))
    with smtplib.SMTP(email_settings.get("host", "localhost")) as smtp_server:
        smtp_server.sendmail(email_settings["sender"], recipient, msg.as_string())


def forge_forgotten_email(username: str, email: str, token: str) -> Tuple[str, str]:
    """Return email template
    """
    del email
    del token
    return (
        f"""
        Salut {username},

        Un mot de passe est si vite oublié !
        Une demande de récuperation de mot de passe à été enregistrée.
    """,
        f"""
    <p>Salut {username},</p>
    <p>
        Un mot de passe est si vite oublié !
        Une demande de récuperation de mot de passe à été enregistrée.
    </p>
    """,
    )



import pytest

import kisee.kisee as kisee


@pytest.fixture
def client(loop, aiohttp_client):
    settings = kisee.load_conf("tests/test_settings.toml")
    app = kisee.identification_app(settings)
    return loop.run_until_complete(aiohttp_client(app))


async def test_bad_json(client):
    response = await client.request(
        "POST",
        "/users/",
        data=b'{"login" "my_login"}',
        headers={"Content-Type": "application/json"},
    )
    assert response.status == 400



import smtplib
from typing import List, Tuple, Dict

from kisee.emails import send_mail


class MockSMTP:
    sent_emails: List[Tuple[List, Dict]] = []

    def __init__(self, *args, **kwargs):
        super().__init__()

    def __enter__(self):
        MockSMTP.sent_emails = []
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        ...

    def sendmail(self, *args, **kwargs):
        self.sent_emails.append((args, kwargs))


def test_send_mail(monkeypatch):
    monkeypatch.setattr(smtplib, "SMTP", MockSMTP)
    send_mail(
        "Subject",
        "texttexttext",
        "htmlhtmlhtml",
        {"sender": "bar@localhost"},
        "foo@localhost",
    )
    assert len(MockSMTP.sent_emails) == 1
    assert MockSMTP.sent_emails[0][0][1] == "foo@localhost"



"""Serialisers using coreapi, the idea is to (in the future) provide
various representations of our resources like mason, json-ld, hal, ...

"""
import coreapi
from aiohttp import web


def serialize(
    request: web.Request, document: dict, status=200, headers=None
) -> web.Response:
    """Serialize the given document according to the Accept header of the
    given request.
    """
    accept = request.headers.get("Accept")
    codec = coreapi.utils.negotiate_encoder([coreapi.codecs.CoreJSONCodec()], accept)
    content = codec.dump(document, indent=True)
    return web.Response(
        body=content, content_type=codec.media_type, headers=headers, status=status
    )



"""Mocks for kisee service
"""
from kisee.identity_provider import UserAlreadyExist


def register_user(self, username, password, is_superuser=False):
    """Mocker register user
    """
    raise UserAlreadyExist


def send_mail(subject, text, html, email_settings, recipient):
    """Send mail
    """
    pass



"""Authentication handler
"""
import base64
import binascii
from typing import Mapping, Tuple, Union

import jwt
from aiohttp import web

from kisee.identity_provider import IdentityProvider, User

Claims = Mapping[str, Union[str, bool, int]]


async def _basic_authentication(
    encoded: bytes, idp: IdentityProvider
) -> Tuple[User, Claims]:
    """Authentication using Basic scheme.
    """
    try:
        decoded = base64.b64decode(encoded)
    except binascii.Error:
        raise web.HTTPUnauthorized(reason="Bad authorization")
    try:
        username, password = decoded.decode("utf-8").split(":", 1)
    except ValueError:
        raise web.HTTPUnauthorized(reason="Bad authorization")
    user = await idp.identify(username, password)
    if user is None:
        raise web.HTTPUnauthorized(reason="Bad authorization")
    # Using basic auth means user knows its password so he's authorized to change it.
    return user, {"can_change_pwd": True}


async def _jwt_authentication(
    token: str, idp: IdentityProvider, public_key: str
) -> Tuple[User, Claims]:
    """Authentication using JWT.
    """
    try:
        claims = jwt.decode(token, public_key, algorithms="ES256")
    except jwt.DecodeError as err:
        raise web.HTTPUnauthorized(reason="Bad authorization") from err
    else:
        return await idp.get_user_by_username(claims["sub"]), claims


async def authenticate_user(request: web.Request) -> Tuple[User, Claims]:
    """Multiple schemes authentication using request Authorization header.
    Raises HTTPUnauthorized on failure.
    """
    if not request.headers.get("Authorization"):
        raise web.HTTPUnauthorized(reason="Missing authorization header")
    scheme, value = request.headers.get("Authorization").strip().split(" ", 1)
    if scheme == "Basic":
        return await _basic_authentication(value, request.app["identity_backend"])
    if scheme == "Bearer":
        return await _jwt_authentication(
            value,
            request.app["identity_backend"],
            request.app["settings"]["jwt"]["public_key"],
        )
    raise web.HTTPUnauthorized(reason="Bad authorization")



"""Postgresql backend for Kisee
"""
import uuid
from typing import Optional

import hmac
import bcrypt
import asyncpg

from kisee.identity_provider import IdentityProvider, User, UserAlreadyExist


def constant_time_compare(val1: bytes, val2: bytes) -> bool:
    """Return True if the two strings are equal, False otherwise."""
    return hmac.compare_digest(val1, val2)


def verify(user_input: bytes, database_encoded: bytes) -> bool:
    """Verity that the given encoded (hashed) password is matching the
    expected password.

    Needs the bcrypt library.  Please be warned that this library
    depends on native C code and might cause portability issues.

    This is compatible with Python Blowfish cipher (prefix is '$2b$').
    """
    if not database_encoded.startswith(b"$2b$"):
        raise ValueError(
            "The encoded string must be Python Blowfish cipher compatible."
        )
    encoded_user_input = bcrypt.hashpw(user_input, database_encoded)
    return constant_time_compare(database_encoded, encoded_user_input)


class DataStore(IdentityProvider):
    """Postgresql backend for kisee
    """

    def __init__(self, options: dict, **kwargs) -> None:
        super().__init__(options, **kwargs)
        self.user = options["user"]
        self.password = options["password"]
        self.database = options["database"]
        self.host = options["host"]
        self.port = options["port"]

    async def __aenter__(self):
        self.pool = await asyncpg.create_pool(  # pylint: disable=W0201
            database=self.database,  # W0201 is attribute-defined-outside-init
            user=self.user,  # we define it outside of init on purpose
            password=self.password,
            host=self.host,
            port=self.port,
        )

    async def __aexit__(self, exc_type, exc_value, traceback):
        try:
            self.pool.terminate()
        except AttributeError:
            pass

    async def identify(self, login: str, password: str) -> Optional[User]:
        """Identifies the given login/password pair, returns a dict if found.
        """
        async with self.pool.acquire() as connection:
            result = await connection.fetchrow(
                "SELECT * FROM users WHERE (username = $1 OR email = $1) ", login
            )
            if verify(password.encode("utf-8"), result["password"].encode("utf-8")):
                return User(result["user_id"], result["username"], result["is_superuser"])
        return None

    async def register_user(
        self, username: str, password: str, email: str, is_superuser: bool = False
    ):
        password_hashed = bcrypt.hashpw(password.encode(), bcrypt.gensalt())
        user_id = str(uuid.uuid4())
        async with self.pool.acquire() as connection:
            try:
                await connection.execute(
                    """
                    INSERT INTO users(
                        user_id, username, password, email, is_superuser
                    ) VALUES (
                        $1, $2, $3, $4, $5
                    )
                """,
                    user_id,
                    username,
                    password_hashed.decode("utf-8"),
                    email,
                    is_superuser,
                )
            except asyncpg.exceptions.UniqueViolationError:
                    raise UserAlreadyExist

    async def get_user_by_email(self, email):
        """Get user with provided email address
        """
        async with self.pool.acquire() as connection:
            result = await connection.fetchrow(
                """SELECT * FROM users WHERE email = $1""",
                email
            )
        user_data = dict(result)
        del user_data["password"]
        return User(**user_data)

    async def get_user_by_username(self, username):
        """Get user with provided username
        """
        async with self.pool.acquire() as connection:
            result = await connection.fetchrow(
                """"SELECT * FROM users WHERE username = $1""",
                username
            )
        user_data = dict(result)
        del user_data["password"]
        return User(**user_data)

    async def set_password_for_user(self, user: User, password: str):
        async with self.pool.acquire() as connection:
            await connection.execute(
                """
                    UPDATE users
                    SET password = $1
                    WHERE username = $2
                """,
                password,
                user.username
            )

    async def is_connection_alive(self) -> bool:
        """Verify that connection is alive, always return True
        """
        try:
            async with self.pool.acquire() as connection:
                connection.execute("SELECT 1")
                return True
        except AttributeError:
            return False



"""Kisee quickstart"""

from subprocess import run, PIPE
from typing import Callable, Any, Dict, Union, Optional

import toml

ConfigurationValue = Union[str, int, bool]
Config = Dict[str, Dict[str, ConfigurationValue]]


def input_or_default(
    prompt: str,
    default: Optional[str] = None,
    validator: Callable[[str], ConfigurationValue] = lambda x: x,
) -> ConfigurationValue:
    """The validator is expected to either convert to the expected type,
    or raise a ValueError.

    Optional shortcut: If [] is present in prompt, it will be replaced
    by the default value.
    """
    if default:
        validator(default)  # Fail loudly if the default don't pass the validator.
    while True:
        response = input(prompt.replace("[]", f"[{default}]"))
        if default is not None and not response:
            return validator(default)
        try:
            return validator(response)
        except ValueError as err:
            print(err)
            continue


def boolean(value: str) -> bool:
    """Convert a "human" boolean (Y, Yes, True, ...) to a Python boolean,
    raising a ValueError if it can't be converted.
    """
    if value.lower() not in {"y", "yes", "n", "no", "true", "false"}:
        raise ValueError('Please enter either "y" or "n".')
    return value.lower() in {"y", "yes", "true"}


def questions() -> Dict[str, Any]:
    """Ask questions interactively to the user to build a settings file.
    """
    print(
        "Welcome to the Kisee quickstart configurator,",
        "Just press ENTER to accept a default value (if given, in brackets)",
        sep="\n",
    )
    responses: Config = {
        "server": {},
        "identity_backend": {"class": "kisee.providers.demo.DemoBackend"},
        "jwt": {"iss": "example.com"},
    }
    responses["server"]["host"] = input_or_default(f"Listen to []: ", "127.0.0.1")
    responses["server"]["port"] = input_or_default(f"Listen on port []: ", "8140")
    responses["server"]["hostname"] = input_or_default(
        f"Exposed URL []: ", "http://localhost:8140"
    )
    responses["server"]["debug"] = input_or_default(
        f"Debug mode []: ", "No", validator=boolean
    )
    return responses


def add_ec_keys(config: Config) -> None:
    """Invokes openssl to create a new secp256k1 key pair.
    """
    private_key_process = run(
        ["openssl", "ecparam", "-name", "secp256k1", "-genkey", "-noout", "-out", "-"],
        check=True,
        stdout=PIPE,
    )
    private_key = private_key_process.stdout.decode("ASCII")
    public_key_process = run(
        ["openssl", "ec", "-in", "-", "-pubout"],
        check=True,
        input=private_key_process.stdout,
        stdout=PIPE,
    )
    public_key = public_key_process.stdout.decode("ASCII")
    config["jwt"]["private_key"] = private_key
    config["jwt"]["public_key"] = public_key


def main() -> None:
    """Kisee-quickstart entry point.
    """
    config = questions()
    add_ec_keys(config)
    print("Writing configuration file to settings.toml...")
    with open("settings.toml", "w") as settings_file:
        toml.dump(config, settings_file)


if __name__ == "__main__":
    main()



from kisee import kisee
import base64
from datetime import datetime, timedelta
import pytest
import asyncio
import aiohttp
import jwt
from kisee.providers.demo import DemoBackend
from hypothesis import given
from hypothesis.strategies import text, binary

from kisee.authentication import _jwt_authentication, _basic_authentication


@pytest.fixture
def settings():
    return kisee.load_conf("tests/test_settings.toml")


@pytest.fixture
def valid_token(settings):
    return jwt.encode(
        {
            "iss": "Pouete",
            "sub": "toto",
            "exp": datetime.utcnow() + timedelta(hours=1),
            "jti": "42",
        },
        settings["jwt"]["private_key"],
        algorithm="ES256",
    ).decode("utf8")


def run(coro):
    return asyncio.get_event_loop().run_until_complete(coro)


@given(text())
def test_jwt_authentication_fails(settings, s):
    """Assert that a random string is never a valid token.
    """
    with pytest.raises(aiohttp.web_exceptions.HTTPUnauthorized):
        run(_jwt_authentication(s, DemoBackend(), settings["jwt"]["public_key"]))


def test_jwt_authentication_succeed(settings, valid_token):
    user, claims = run(
        _jwt_authentication(valid_token, DemoBackend(), settings["jwt"]["public_key"])
    )
    assert user.username == "toto"


@given(binary())
def test_basic_authentication_fails(s: bytes):
    """Assert that a random string is never a valid basic auth.
    """
    with pytest.raises(aiohttp.web_exceptions.HTTPUnauthorized):
        run(_basic_authentication(s, DemoBackend()))


def test_basic_authentication_succeed():
    user, claims = run(
        _basic_authentication(base64.b64encode(b"toto:toto"), DemoBackend())
    )
    assert user.username == "toto"


def test_basic_authentication_bad_password():
    with pytest.raises(aiohttp.web_exceptions.HTTPUnauthorized):
        user, claims = run(
            _basic_authentication(base64.b64encode(b"toto:t"), DemoBackend())
        )



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['kisee', 'kisee.providers']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp',
 'aiomysql',
 'asyncpg',
 'bcrypt',
 'coreapi',
 'cryptography',
 'psutil',
 'pyjwt',
 'sentry-sdk',
 'shortuuid',
 'toml']

extras_require = \
{'dev': ['hypothesis',
         'pycalver',
         'pytest',
         'pytest-cov',
         'pytest-aiohttp',
         'flake8',
         'pylint',
         'black',
         'bandit',
         'mypy']}

entry_points = \
{'console_scripts': ['kisee = kisee.kisee:main',
                     'kisee-quickstart = kisee.quickstart:main']}

setup(name='kisee',
      version='19.11.0',
      description='HTTP server managing identities.',
      author='Julien Palard',
      author_email='julien@python.org',
      url='https://github.com/meltygroup/kisee',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
